# Copyright 2018, 2019 Cumulus Networks, Inc.  All Rights reserved.
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from __future__ import absolute_import
import psutil
from netq_lib.orm.redisdb.models import ResourceUtil
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate


def process_resource_util(cmd_output, svc_failed=False, **kwargs):

    fullstate = kwargs.get('fullstate')
    cpu_util = psutil.cpu_percent()
    mem_percent_util = psutil.virtual_memory().percent
    new_resource_util_obj = ResourceUtil()
    new_resource_util_obj.cpu_utilization = cpu_util
    new_resource_util_obj.mem_utilization = mem_percent_util

    disk_mode = cmd_output.split(',')[0]
    if disk_mode == 'ro':
        new_resource_util_obj.is_disk_read_only = True
    else:
        new_resource_util_obj.is_disk_read_only = False

    disk_partitions = psutil.disk_partitions(all=False)
    disk_utilization_json = {}
    for sdiskpart in disk_partitions:
        disk_path = sdiskpart.device
        disk_usage = psutil.disk_usage(sdiskpart.mountpoint)
        disk_utilization_json[disk_path] = {
            'percent': disk_usage.percent,
            'total': disk_usage.total,
            'used': disk_usage.used
        }

    new_resource_util_obj.disk_utilization = disk_utilization_json

    new_resource_util_obj.save()

    resource_state_arr = []
    resource_state_arr.append(new_resource_util_obj)
    if(fullstate is True):
        save_fullstate(resource_state_arr, ResourceUtil().__class__.__name__)

    return ServiceStatus.OK
